function determine_slopes(dir_name_part, fl_pixels)
% function determine_slopes(dir_name_part, fl_pixels)

% if SLOPES_IN_DEGREES is false, the slopes will be plotted without unit
SLOPES_IN_DEGREES = true;

% if LOAD_DATA is true, just load the flow_data.mat file (no vision
% processing)
LOAD_DATA = false;

if(~exist('dir_name_part', 'var') || isempty(dir_name_part))
    dir_name_part = 'pure_ver';
end
if(~exist('fl_pixels', 'var') || isempty(fl_pixels))
    n_pixels_camera = 256;
    FOV = 50;
    fl_pixels = 0.5 * n_pixels_camera / tan(0.5 * deg2rad(FOV));
end

% extract the directories with dir_name_part as part of their directory
% name:
dir_pref = './';
prefix_dirs = makeListOfDirs(dir_pref, dir_name_part);

% To order the directories alphabetically / numerically, use the following script:
% http://www.mathworks.com/matlabcentral/fileexchange/10959-sortnat-natural-order-sort
if(exist('sort_nat', 'file'))
    prefix_dirs = sort_nat(prefix_dirs);
end

n_dirs = length(prefix_dirs);
if(LOAD_DATA)
    load('flow_data');
else
    flow_data = cell(n_dirs, 1);
    for d = 1:n_dirs
        fprintf('Processing directory: %s\n', prefix_dirs{d});
        flow_data{d} = test_flow(prefix_dirs{d});
    end 
    save('flow_data', 'flow_data');
end

% Show alpha slopes (in x-direction)
h(1) = figure();
set(gcf, 'Color', [1 1 1]);
hold on;
if(SLOPES_IN_DEGREES)
    f = 0.5 * n_pixels_camera / tan(0.5 * deg2rad(FOV));
end
for d = 1:n_dirs
    if(SLOPES_IN_DEGREES)
        plotZx = rad2deg(atan(fl_pixels*smooth(flow_data{d}.Zx(:,1))));
    else
        plotZx = smooth(flow_data{d}.Zx(:,1));
    end
    plot(1:length(plotZx), plotZx, 'LineWidth', 2, 'Color', (d / n_dirs) * [1 0 0]);
end
xlabel('Time (frames)');
ylabel('Slope \alpha');
set(h(1), 'Name', 'Slope alpha',  'NumberTitle', 'off');

% Show beta slopes (in y-direction)
h(2) = figure();
set(gcf, 'Color', [1 1 1]);
hold on;
for d = 1:n_dirs
    if(SLOPES_IN_DEGREES)
        plotZy = rad2deg(atan(fl_pixels*smooth(flow_data{d}.Zy(:,1))));
    else
        plotZy = smooth(flow_data{d}.Zy(:,1));
    end
    plot(1:length(plotZy), plotZy, 'LineWidth', 2, 'Color', (d / n_dirs) * [1 0 0]);
end
xlabel('Time (frames)');
ylabel('Slope \alpha');
set(h(2), 'Name', 'Slope beta',  'NumberTitle', 'off');

% Show time-to-contact:
h(3) = figure();
set(gcf, 'Color', [1 1 1]);
hold on;
for d = 1:n_dirs
    plotTTC = flow_data{d}.TTC;
    plot(1:length(plotTTC), plotTTC, 'LineWidth', 2, 'Color', (d / n_dirs) * [0 0 1]);
end
xlabel('Time (frames)');
ylabel('Unfiltered time-to-contact');
set(h(3), 'Name', 'Unfiltered time-to-contact',  'NumberTitle', 'off');

% Ventral flow:
h(4) = figure;
set(gcf, 'Color', [1 1 1]);
hold on;
for d = 1:n_dirs
    plotOX = flow_data{d}.OmegaX;
    plotOY = flow_data{d}.OmegaY;
    plot(1:length(plotOX), plotOX, 'LineWidth', 2, 'Color', (d / n_dirs) * [1 0 1]);
    plot(1:length(plotOY), plotOY, 'LineWidth', 2, 'Color', (d / n_dirs) * [0 1 0]);
end
xlabel('Time (frames)');
ylabel('Unfiltered ventral flow');
set(h(4), 'Name', 'Unfiltered ventral flow',  'NumberTitle', 'off');

% Inliers
h(5) = figure;
set(gcf, 'Color', [1 1 1]);
hold on;
for d = 1:n_dirs
    plotInliers = flow_data{d}.inliers;
    plot(1:length(plotInliers), plotInliers, 'LineWidth', 2, 'Color', (d / n_dirs) * [0 1 1]);
end
xlabel('Time (frames)');
ylabel('Inliers RANSAC');
set(h(5), 'Name', 'Inliers RANSAC',  'NumberTitle', 'off');

    
% Errors
h(6) = figure;
set(gcf, 'Color', [1 1 1]);
hold on;
for d = 1:n_dirs
    plotErrors = flow_data{d}.errors;
    plot(1:length(plotErrors), plotErrors, 'LineWidth', 2, 'Color', (d / n_dirs) * [0 1 1]);
end
xlabel('Time (frames)');
ylabel('Errors RANSAC');
set(h(6), 'Name', 'Errors RANSAC',  'NumberTitle', 'off');