function [pu, pv, inliersv, inliersu, error_u, error_v, rank_deficient] = estimateFlowPlanes(X, V, n_matches, WIDTH, HEIGHT)

RANSAC = false;
pu = []; pv = [];
inliersu = []; inliersv = [];
error_u = []; error_v = [];

rank_deficient = false;

if(~RANSAC)
    A = [X ones(n_matches,1)];
    b = V(:,1);
    pu = A\b;
    
    % refit:
    err = A * pu - b;
    err = err .* err;
    low_err = find(err < median(err));
    n_low_err = length(low_err);
    A = [X(low_err,:), ones(n_low_err,1)];
    b = V(low_err,1);
    if(rank(A) < 3)
        rank_deficient = true;
    end
    pu = A \ b;
    error_u = mean(abs(A * pu - b));
    inliersu = n_low_err / n_matches;
    
    % then the slope for v (flow_y):
    A = [X ones(n_matches,1)];
    b = V(:,2);
    pv = A\b;
    
    % refit:
    err = A * pv - b;
    err = err .* err;
    low_err = find(err < median(err));
    n_low_err = length(low_err);
    A = [X(low_err,:), ones(n_low_err,1)];
    b = V(low_err,2);
    if(rank(A) < 3)
        rank_deficient = true;
    end
    pv = A \ b;
    error_v = mean(abs(A * pv - b));
    inliersv = n_low_err / n_matches;
else
    n_iterations = 20;
    n_samples = min([5,n_matches]); % minimum is 3
    if(n_samples < 3)
        % pu = [1 0 0];
        % pv = [0 1 0];
        fprintf('Not enough features...\n');
        return;
    end
    threshold = 0.5;
    % first for dx
    inliersu = zeros(n_samples, 1);
    errsu = zeros(n_samples, 1);
    PU = zeros(3, n_samples);
    AA = [X, ones(n_matches,1)];
    bb = V(:,1);
    if(rank(AA) < 3)
        rank_deficient = true;
    end
    for s = 1:n_iterations
        inds = randperm(n_matches);
        inds = inds(1:n_samples);
        % solve the small system
        A = [X(inds,:) ones(n_samples,1)];
        b = V(inds,1);
        pu = A\b; % we get rank deficient errors here...
        errs = abs(AA * pu - bb);
        inl = find(errs < threshold);
        % gather statistics:
        inliersu(s) = length(inl) / n_matches;
        errsu(s) = mean(errs(inl));
        PU(:,s) = pu;
    end
    %     [error_u,i] = min(errsu);
    %     pu = PU(:,i);
    %     inliersu = inliersu(i);
    [inliersu, i] = max(inliersu);
    pu = PU(:,i);
    error_u = mean(abs(AA * pu - bb));% errsu(i);
    
    % then for dy
    inliersv = zeros(n_samples, 1);
    errsv = zeros(n_samples, 1);
    PV = zeros(3, n_samples);
    AA = [X, ones(n_matches,1)];
    bb = V(:,2);
    if(rank(AA) < 3)
        rank_deficient = true;
    end
    for s = 1:n_samples
        inds = randperm(n_matches);
        inds = inds(1:n_samples);
        % solve the small system
        A = [X(inds,:) ones(n_samples,1)];
        b = V(inds,2);
        pv = A\b;
        errs = abs(AA * pv - bb);
        inl = find(errs < threshold);
        % gather statistics:
        inliersv(s) = length(inl) / n_matches;
        errsv(s) = mean(errs(inl));
        PV(:,s) = pv;
    end
    %     [error_v,i] = min(errsv);
    %     pv = PV(:,i);
    %     inliersv = inliersv(i);
    [inliersv, i] = max(inliersv);
    pv = PV(:,i);
    error_v = mean(abs(AA * pv - bb)); %errsv(i);
end
