function [pu, pv, inliersv, inliersu, error_u, error_v, rank_deficient] = estimateFlowQuadratic(X, V, n_matches, WIDTH, HEIGHT)

RANSAC = true;
% when REGULARIZED = true, the quadratic terms are assumed to be small,
% favoring more linear fits:
REGULARIZED = false;
lambda = 4;
LAMBDA = zeros(5);
LAMBDA(4,4) = lambda;
LAMBDA(5,5) = lambda;

% variable initialization:
pu = []; pv = [];
inliersu = []; inliersv = [];
error_u = []; error_v = [];

rank_deficient = false;

if(~RANSAC)
    % u = pu(1) + pu(2) * x + pu(3) * y + pu(4) * x^2 + pu(5) * xy
    A = [ones(n_matches,1), X, X(:,1).*X(:,1), X(:,1).*X(:,2)];
    b = V(:,1);
    if(REGULARIZED)
        pu = inv(LAMBDA + A' * A) * A' * b;
    else
        pu = A \ b;
    end
    
    % refit:
    err = A * pu - b;
    err = err .* err;
    low_err = find(err < median(err));
    n_low_err = length(low_err);
    A = [ones(n_low_err,1), X(low_err,:), X(low_err,1).*X(low_err,1), X(low_err,1).*X(low_err,2)];
    b = V(low_err,1);
    if(rank(A) < 3)
        rank_deficient = true;
    end
    if(REGULARIZED)
        pu = inv(LAMBDA + A' * A) * A' * b;
    else
        pu = A \ b;
    end
    error_u = mean(abs(A * pu - b));
    inliersu = n_low_err / n_matches;
    
    % then the slope for v (flow_y):
    % v = pv(1) + pv(2) * x + pv(3) * y + pv(4) * y^2 + pv(5) * xy
    A = [ones(n_matches,1), X, X(:,2).*X(:,2), X(:,1).*X(:,2)];
    b = V(:,2);
    if(REGULARIZED)
        pv = inv(LAMBDA + A' * A) * A' * b;
    else
        pv = A\b;
    end
    
    % refit:
    err = A * pv - b;
    err = err .* err;
    low_err = find(err < median(err));
    n_low_err = length(low_err);
    A = [ones(n_low_err,1), X(low_err, :), X(low_err,2).*X(low_err,2), X(low_err,1).*X(low_err,2)];
    % A = [X(low_err,:), ones(n_low_err,1)];
    b = V(low_err,2);
    if(rank(A) < 3)
        rank_deficient = true;
    end
    if(REGULARIZED)
        pv = inv(LAMBDA + A' * A) * A' * b;
    else
        pv = A \ b;
    end
    error_v = mean(abs(A * pv - b));
    inliersv = n_low_err / n_matches;
else
    n_iterations = 30;
    n_samples = min([5,n_matches]); % minimum is 4
    if(n_samples < 5)
        fprintf('Not enough features...\n');
        return;
    end
    threshold = 0.5;
    
    % first for dx
    % u = pu(1) + pu(2) * x + pu(3) * y + pu(4) * x^2 + pu(5) * xy
    inliersu = zeros(n_iterations, 1);
    errsu = zeros(n_iterations, 1);
    PU = zeros(5, n_iterations);
    AA = [ones(n_matches,1), X, X(:,1).*X(:,1), X(:,1).*X(:,2)];
    bb = V(:,1);
    if(rank(AA) < 5)
        rank_deficient = true;
    end
    for s = 1:n_iterations
        inds = randperm(n_matches);
        inds = inds(1:n_samples);
        % solve the small system
        A = [ones(n_samples,1), X(inds,:), X(inds,1) .* X(inds,1), X(inds,1) .* X(inds,2)];
        b = V(inds,1);
        if(REGULARIZED)
            pu = pinv(LAMBDA + A' * A) * A' * b;
        else 
            if(rank(A) < 5)
                pu = zeros(5, 1); % is this the right solution?
            else 
                pu = A\b; 
            end
        end
        errs = abs(AA * pu - bb);
        inl = find(errs < threshold);
        % gather statistics:
        inliersu(s) = length(inl) / n_matches;
        errsu(s) = mean(errs(inl));
        PU(:,s) = pu;
    end
    %     [error_u,i] = min(errsu);
    %     pu = PU(:,i);
    %     inliersu = inliersu(i);
    [inliersu, i] = max(inliersu);
    pu = PU(:,i);
    error_u = mean(abs(AA * pu - bb));%errsu(i);
    
    % then for dy
    % v = pv(1) + pv(2) * x + pv(3) * y + pv(4) * y^2 + pv(5) * xy
    inliersv = zeros(n_iterations, 1);
    errsv = zeros(n_iterations, 1);
    PV = zeros(5, n_iterations);
    AA = [ones(n_matches,1), X, X(:,2).*X(:,2), X(:,1).*X(:,2)];
    bb = V(:,2);
    if(rank(AA) < 5)
        rank_deficient = true;
    end
    for s = 1:n_samples
        inds = randperm(n_matches);
        inds = inds(1:n_samples);
        % solve the small system
        A = [ones(n_samples,1), X(inds,:), X(inds,2).*X(inds,2), X(inds,1).*X(inds,2)];
        b = V(inds,2);
        if(REGULARIZED)
            pv = pinv(LAMBDA + A' * A) * A' * b;
        else
            pv = A\b;
        end
        errs = abs(AA * pv - bb);
        inl = find(errs < threshold);
        % gather statistics:
        inliersv(s) = length(inl) / n_matches;
        errsv(s) = mean(errs(inl));
        PV(:,s) = pv;
    end
    %     [error_v,i] = min(errsv);
    %     pv = PV(:,i);
    %     inliersv = inliersv(i);
    [inliersv, i] = max(inliersv);
    pv = PV(:,i);
    error_v =  mean(abs(AA * pv - bb));%errsv(i);
end
