function G = getG(Ix, Iy, x, y, window_size)
% function G = getG(Ix, Iy, x, y, window_size)
% G = [Ixx, Ixy; Ixy, Iyy], summed over the entire window
%
% (c) Guido de Croon, 2013
[H,W] = size(Ix);
if(~exist('x', 'var') || sum(size(x)) < 2)
    x = window_size + 1;
end
if(~exist('y', 'var') || sum(size(y)) < 2)
    y = window_size + 1;
end
if(x <= window_size || x > W - window_size)
    error('x out of range\n');
end
if(y <= window_size || y > H - window_size)
    error('y out of range\n');
end

Ax = Ix(y-window_size:y+window_size, x-window_size:x+window_size);
Ay = Iy(y-window_size:y+window_size, x-window_size:x+window_size);

% remove very small values from Ax and Ay
epsilon = 1E-5;
Mx = Ax >= epsilon;
Ax = Ax .* Mx;
My = Ay >= epsilon;
Ay = Ay .* My;

% determine the components used for the G-matrix
Axx = Ax .* Ax;
Axy = Ax .* Ay;
Ayy = Ay .* Ay;

% determine G:
sAxy = sum(Axy(:));
G = [sum(Axx(:)), sAxy; sAxy, sum(Ayy(:))];
