function NextPyramid = makeNextPyramid(Pyramid)
% function NextPyramid = makeNextPyramid(Pyramid)
% (c) Guido de Croon, 2013
SIMPLE_RESIZE = false;

if(SIMPLE_RESIZE)
    % this is done with bicubic sampling (4x4 neighbourhood):
    NextPyramid = imresize(Pyramid, 0.5);
else
    filter = [1/16, 1/4, 3/8, 1/4, 1/16];
    filter = filter' * filter; % 5 x 5
    filter_size = size(filter,1);
    half_size = floor(filter_size / 2);
    % Height and width of the pyramid image
    H = size(Pyramid,1);
    W = size(Pyramid,2);
    NextPyramid = zeros(floor(H/2), floor(W/2));
    % pad the original image:
    Pyramid = [repmat(Pyramid(1,:), [half_size, 1]); Pyramid; repmat(Pyramid(H,:), [half_size, 1])];
    Pyramid = [repmat(Pyramid(:,1), [1, half_size]), Pyramid, repmat(Pyramid(:,W), [1, half_size])];
    % xx, yy are the coordinates of the next pyramid level
    xx = 1;
    % x,y are the coordinated of the current pyramid level
    for x = 2+half_size:2:W+half_size
        yy = 1;
        for y = 2+half_size:2:H+half_size
            Filtered = filter .* Pyramid(y-half_size:y+half_size, x-half_size:x+half_size);
            NextPyramid(yy,xx) = sum(Filtered(:));
            yy = yy + 1;
        end
        xx = xx + 1;
    end
end
